// TODO: added the next three lines to load the prototype library the Node.js way
var prototype_lib = require('../prototype-minor/prototype.js');
var $A = prototype_lib.dollar_A;
var $H = prototype_lib.dollar_H;

var Ersatz = exports;

Ersatz.expectations = [];

Ersatz.mock = function(obj) {
    return Object.extend(obj, Ersatz.Extensions);
  };

Ersatz.verifyExpectations = function(failFunction) {
  try {
    this.expectations.invoke('verify', failFunction);
  } finally {
    this.expectations.clear();
  }
};


Ersatz.Extensions = {
  expects: function(methodName) {
    return new Ersatz.Expectation(this, methodName);
  },
  
  stubs: function(methodName) {
    return new Ersatz.Stub(this, methodName);    
  }
};

Ersatz.Stub = Class.create({
  initialize: function(obj, methodName) {
    this.object = obj;
    this.methodName = methodName;
    this.calls = $A();
    this.returnValue = $A();
    Ersatz.expectations.push(this);
    this.origMethod = this.object[methodName];
    this._replaceOriginalMethod();
  },
  
  returns: function() {
    this.returnValue = $A(arguments);
    return this;
  },
  
  returning: function(returnFunction) {
    this.returningFunction = returnFunction;
    return this;
  },
  
  verify: function(failFunc) {
    this.object[this.methodName] = this.origMethod;  
  },
  
// Private
  _returnValue: function(args) {
    if (this.returningFunction) {
      return this.returningFunction.apply(this, args);
    } else {
      return (this.returnValue.length > 1) ? this.returnValue[this.calls.length - 1] : this.returnValue[0];
    }
   },

  _replaceOriginalMethod: function() {
    this.object[this.methodName] = function() {
      this.calls.push({args: $A(arguments)});
      return this._returnValue(arguments);
    }.bind(this);
  }
  
});

Ersatz.Expectation = Class.create(Ersatz.Stub, {
  initialize: function($super, obj, methodName) {
    $super(obj, methodName)
    this.expectedCallCounts = $H();
  },
  
  with: function(){
    this.expectedArgs = $A(arguments);
    return this;
  },
  
  with_expression: function(exprFunction){
    this.expectedArgsFunc = exprFunction;
    return this;
  },
  
  at_least: function(count){
    this.expectedCallCounts.set('AtLeast', count);
    return this;    
  },

  at_most: function(count){
    this.expectedCallCounts.set('AtMost', count);
    return this;    
  },
  
  never: function(){
    this.expectedCallCounts.set('Exactly', 0);
    return this;
  },
  
  times: function(count){
    this.expectedCallCounts.set('Exactly', count);
    return this;
  },
  
  verify: function($super, failFunc) {
    try {
      this._verifyCallCount(failFunc);
      this._verifyCallArguments(failFunc);
    } finally {
      $super();
    }
  },
  
// Private
  
  _hasCallCount: {
    Exactly: function(times) {
      return this.calls.length == times;
    },

    AtLeast: function(times) {
      return this.calls.length >= times;
    },

    AtMost: function(times) {
      return this.calls.length <= times;
    }
  },
  
  _verifyCallCount: function(failFunc){
    // Use a default if no explicit counts were specified
    if (this.expectedCallCounts.keys().length == 0) { this.expectedCallCounts.set('Exactly', 1) }
    this.expectedCallCounts.each(function(pair){
      if (this['_hasCallCount'][pair.key].call(this, pair.value)) { return true; }
      failFunc('Expected '+this.methodName+' to be called '+pair.key.toLowerCase()+' '+pair.value+' times.');
    }, this)
  },
    
  _verifyCallArguments: function(failFunc) {
    // FIXME - this  logic is grody. genericize this stuff to one set of tests.
    if (!this.expectedArgs && !this.expectedArgsFunc) { return true; }
    if (this.expectedArgsFunc) {
      var expectedArgs = 'Result of function: ' + this.expectedArgsFunc.toString();
      var matcher = function(callVals) { return this.expectedArgsFunc.apply(this, callVals.args); }
    } else {
      var expectedArgs = this.expectedArgs.inspect();
      var matcher = function(callVals) { return Object.match(this.expectedArgs, callVals.args); }
    }
    
    var match = this.calls.detect(matcher, this)
    if (match) { return true; }
    failFunc('Expected '+this.methodName+' to get called with: '+expectedArgs+' but was called with: '+this.calls.inspect()+'.');
  }
  
});

Ersatz.Expectation.prototype.atLeast = Ersatz.Expectation.prototype.at_least;
Ersatz.Expectation.prototype.atMost = Ersatz.Expectation.prototype.at_most;
Ersatz.Expectation.prototype.having = Ersatz.Expectation.prototype['with'];

// FIXME: This probably doesn't work for Elements or Dates
// TODO: refactor this to be like Object.ptototype.__equal__ so we can define __equal__ on other objects
Object.match = function(obj1, obj2) {
  if (typeof(obj1) != typeof(obj2)) { return false; }
  // make this === after you add a test for Object.match(undefined, null);
  if (obj1 == null) { return true; }
  if ( !Object.isElement(obj1) && (typeof(obj1) == 'object')) {
    if (obj1.length != obj2.length) { return false; }

    for(var key in obj1) {
      var match = Object.match(obj1[key], obj2[key]);
      if (!match) { return false; }
    }
    return true;
  }
  return(obj1 == obj2);
};
